window.Gleam.initSearch([{"doc":"funtil","title":"funtil","content":"# funtil\n\n[![Package Version](https://img.shields.io/hexpm/v/funtil)](https://hex.pm/packages/funtil)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/funtil/)\n\nHandy utilities for better functional programming in Gleam!\n\n## Quick start\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\ngleam shell # Run an Erlang shell\n```\n\n## Installation\n\nIf available on Hex this package can be added to your Gleam project:\n\n```sh\ngleam add funtil\n```\n\nand its documentation can be found at &lt;https://hexdocs.pm/funtil&gt;.\n","url":"index.html"},{"doc":"funtil","title":"Never","content":"pub opaque type Never\n This type is used to represent a value that can never happen. What does that\n mean exactly?\n \n - A `Bool` is a type that has two values: `True` and `False`.\n - `Nil` is a type that has one value: `Nil`.\n - `Never` is a type that has zero values: it&#39;s impossible to construct!\n \n Why this type is useful is a bit of a mind-bender, but it&#39;s a useful tool to\n have in your toolbox. For example, if you have a function that returns a\n `Result(Int, Never)` then you *know* that it will always return an `Int` and\n it is safe to use `let assert` to unwrap the value.\n \n\nSynonyms:\nfuntil.Never\nfuntil Never","url":"funtil.html#Never"},{"doc":"funtil","title":"fix","content":"pub fn fix(f: fn(fn(a) -&gt; b, a) -&gt; b) -&gt; fn(a) -&gt; b\n Gleam&#39;s type system does not support recursive `let`-bound functions, even\n though they are theoretically possible. The `fix` combinator is a sneaky way\n around this limitation by making the recursive function a parameter of\n itself.\n \n Sound a bit too magical? Let&#39;s first take a look at what happens if we try\n to write a recursive `let`-bound function in Gleam:\n \n \n pub fn example() {\n   let factorial = fn(x) {\n       case x {\n         0 -&gt; 1\n         x -&gt; x * factorial(x - 1)\n               // ^^^^^^^^^ The name `factorial` is not in scope here.\n       }\n     }\n \n   fact(5)\n   |&gt; should.equal(120)\n }\n \n \n We get a compile error because the name `factorial` is not in scope inside\n the function body. What does it look like if we try to use `fix`?\n \n \n import funtil.{fix}\n \n pub fn example() {\n   let factorial =\n     fix(fn(factorial, x) {\n       case x {\n         0 -&gt; 1\n         x -&gt; x * factorial(x - 1)\n       }\n     })\n \n   fact(5)\n   |&gt; should.equal(120)\n }\n \n \n 🚨 Gleam is designed with this limitation to encourage you to pull things out\n of `let` bindings when they get too complex. If you find yourself reaching for\n `fix`, consider if there&#39;s a clearer way to solve your problem.\n \nSynonyms:\nfuntil.fix\nfuntil fix","url":"funtil.html#fix"},{"doc":"funtil","title":"fix2","content":"pub fn fix2(f: fn(fn(a, b) -&gt; c, a, b) -&gt; c) -&gt; fn(a, b) -&gt; c\n A version of the [`fix`](#fix) util for functions that take two arguments.\n \n 🚨 Gleam is designed with this limitation to encourage you to pull things out\n of `let` bindings when they get too complex. If you find yourself reaching for\n `fix2`, consider if there&#39;s a clearer way to solve your problem.\n \nSynonyms:\nfuntil.fix2\nfuntil fix2","url":"funtil.html#fix2"},{"doc":"funtil","title":"fix3","content":"pub fn fix3(f: fn(fn(a, b, c) -&gt; d, a, b, c) -&gt; d) -&gt; fn(a, b, c) -&gt;\n  d\n A version of the [`fix`](#fix) util for functions that take three arguments.\n \n 🚨 Gleam is designed with this limitation to encourage you to pull things out\n of `let` bindings when they get too complex. If you find yourself reaching for\n `fix3`, consider if there&#39;s a clearer way to solve your problem.\n \nSynonyms:\nfuntil.fix3\nfuntil fix3","url":"funtil.html#fix3"},{"doc":"funtil","title":"never","content":"pub fn never(val: Never) -&gt; a\n If you&#39;ve got a `Never` somewhere in a type it can be a bit of a problem if\n you need something else. Just like [`Never`](#Never) is a type that can never\n be constructed, `never` is a function that can never be called.\n \n To take our `Result(Int, Never)` example from above, what if we want to\n pass that value into a function that expects a `Result(Int, String)`? As it\n stands, the types don&#39;t match up, but because we know that we can never have\n an error, we can use `never` to pretend to convert it into a `String`:\n \n \n import funtil.{Never, never}\n import gleam/io\n import gleam/result\n \n fn log_error(result: Result(a, String)) -&gt; Result(a, String) {\n   case result {\n     Ok(a) -&gt; Nil\n     Error(message) -&gt; io.println(message)\n   }\n \n   result\n }\n \n fn example() {\n   let val: Result(Int, Never) = Ok(42)\n \n   val\n   |&gt; result.map_error(never)\n   |&gt; log_error\n }\n \n \nSynonyms:\nfuntil.never\nfuntil never","url":"funtil.html#never"},{"doc":"funtil","title":"void","content":"pub fn void(_: a) -&gt; Nil\n Take any value and replace it with `Nil`. This can be a nicer way of using\n value-producing functions in places where you only care about their side\n effects.\n\nSynonyms:\nfuntil.void\nfuntil void","url":"funtil.html#void"},{"doc":"funtil","title":"funtil","content":"","url":"funtil.html"}]);